@@ -13,7 +13,7 @@ in Python.
 from __future__ import division
 
 __author__ = "Giampaolo Rodola'"
-__version__ = "2.1.1"
+__version__ = "2.2.1"
 version_info = tuple([int(num) for num in __version__.split('.')])
 
 __all__ = [
@@ -38,28 +38,29 @@ __all__ = [
     "users", "boot_time",                                           # others
 ]
 
-import sys
-import os
-import time
-import signal
-import warnings
+import collections
 import errno
+import functools
+import os
+import signal
 import subprocess
+import sys
+import time
+import warnings
 try:
     import pwd
 except ImportError:
     pwd = None
 
 from psutil._common import memoize
-from psutil._compat import property, callable, defaultdict
-from psutil._compat import (wraps as _wraps,
-                            PY3 as _PY3)
+from psutil._compat import callable, long
+from psutil._compat import PY3 as _PY3
 from psutil._common import (deprecated_method as _deprecated_method,
                             deprecated as _deprecated,
                             sdiskio as _nt_sys_diskio,
                             snetio as _nt_sys_netio)
 
-from psutil._common import (STATUS_RUNNING,
+from psutil._common import (STATUS_RUNNING,  # NOQA
                             STATUS_SLEEPING,
                             STATUS_DISK_SLEEP,
                             STATUS_STOPPED,
@@ -87,16 +88,16 @@ from psutil._common import (CONN_ESTABLISHED,
 
 if sys.platform.startswith("linux"):
     import psutil._pslinux as _psplatform
-    from psutil._pslinux import (phymem_buffers,
+    from psutil._pslinux import (phymem_buffers,  # NOQA
                                  cached_phymem)
 
-    from psutil._pslinux import (IOPRIO_CLASS_NONE,
+    from psutil._pslinux import (IOPRIO_CLASS_NONE,  # NOQA
                                  IOPRIO_CLASS_RT,
                                  IOPRIO_CLASS_BE,
                                  IOPRIO_CLASS_IDLE)
     # Linux >= 2.6.36
     if _psplatform.HAS_PRLIMIT:
-        from _psutil_linux import (RLIM_INFINITY,
+        from _psutil_linux import (RLIM_INFINITY,  # NOQA
                                    RLIMIT_AS,
                                    RLIMIT_CORE,
                                    RLIMIT_CPU,
@@ -136,13 +137,13 @@ if sys.platform.startswith("linux"):
 
 elif sys.platform.startswith("win32"):
     import psutil._pswindows as _psplatform
-    from _psutil_windows import (ABOVE_NORMAL_PRIORITY_CLASS,
+    from _psutil_windows import (ABOVE_NORMAL_PRIORITY_CLASS,  # NOQA
                                  BELOW_NORMAL_PRIORITY_CLASS,
                                  HIGH_PRIORITY_CLASS,
                                  IDLE_PRIORITY_CLASS,
                                  NORMAL_PRIORITY_CLASS,
                                  REALTIME_PRIORITY_CLASS)
-    from psutil._pswindows import CONN_DELETE_TCB
+    from psutil._pswindows import CONN_DELETE_TCB  # NOQA
 
 elif sys.platform.startswith("darwin"):
     import psutil._psosx as _psplatform
@@ -152,7 +153,7 @@ elif sys.platform.startswith("freebsd"):
 
 elif sys.platform.startswith("sunos"):
     import psutil._pssunos as _psplatform
-    from psutil._pssunos import (CONN_IDLE,
+    from psutil._pssunos import (CONN_IDLE,  # NOQA
                                  CONN_BOUND)
 
 else:
@@ -167,6 +168,20 @@ _WINDOWS = os.name == 'nt'
 _timer = getattr(time, 'monotonic', time.time)
 
 
+# Sanity check in case the user messed up with psutil installation
+# or did something weird with sys.path. In this case we might end
+# up importing a python module using a C extension module which
+# was compiled for a different version of psutil.
+# We want to prevent that by failing sooner rather than later.
+# See: https://github.com/giampaolo/psutil/issues/564
+if (int(__version__.replace('.', '')) !=
+        getattr(_psplatform.cext, 'version', None)):
+    msg = "version conflict: %r C extension module was built for another " \
+          "version of psutil (different than %s)" % (_psplatform.cext.__file__,
+                                                     __version__)
+    raise ImportError(msg)
+
+
 # =====================================================================
 # --- exceptions
 # =====================================================================
@@ -251,7 +266,7 @@ def _assert_pid_not_reused(fun):
     """Decorator which raises NoSuchProcess in case a process is no
     longer running or its PID has been reused.
     """
-    @_wraps(fun)
+    @functools.wraps(fun)
     def wrapper(self, *args, **kwargs):
         if not self.is_running():
             raise NoSuchProcess(self.pid, self._name)
@@ -370,7 +385,7 @@ class Process(object):
 
     # --- utility methods
 
-    def as_dict(self, attrs=[], ad_value=None):
+    def as_dict(self, attrs=None, ad_value=None):
         """Utility method returning process information as a
         hashable dictionary.
 
@@ -472,7 +487,7 @@ class Process(object):
         """
         # On POSIX we don't want to cache the ppid as it may unexpectedly
         # change to 1 (init) in case this process turns into a zombie:
-        # https://code.google.com/p/psutil/issues/detail?id=321
+        # https://github.com/giampaolo/psutil/issues/321
         # http://stackoverflow.com/questions/356722/
 
         # XXX should we check creation time here rather than in
@@ -531,8 +546,7 @@ class Process(object):
         if self._exe is None:
             try:
                 exe = self._proc.exe()
-            except AccessDenied:
-                err = sys.exc_info()[1]
+            except AccessDenied as err:
                 return guess_it(fallback=err)
             else:
                 if not exe:
@@ -563,7 +577,12 @@ class Process(object):
                 # might happen if python was installed from sources
                 raise ImportError(
                     "requires pwd module shipped with standard python")
-            return pwd.getpwuid(self.uids().real).pw_name
+            real_uid = self.uids().real
+            try:
+                return pwd.getpwuid(real_uid).pw_name
+            except KeyError:
+                # the uid can't be resolved by the system
+                return str(real_uid)
         else:
             return self._proc.username()
 
@@ -667,13 +686,14 @@ class Process(object):
             else:
                 return self._proc.rlimit(resource, limits)
 
-    # Windows and Linux only
+    # Windows, Linux and BSD only
     if hasattr(_psplatform.Process, "cpu_affinity_get"):
 
         def cpu_affinity(self, cpus=None):
             """Get or set process CPU affinity.
             If specified 'cpus' must be a list of CPUs for which you
             want to set the affinity (e.g. [0, 1]).
+            (Windows, Linux and BSD only).
             """
             if cpus is None:
                 return self._proc.cpu_affinity_get()
@@ -767,7 +787,7 @@ class Process(object):
         else:
             # construct a dict where 'values' are all the processes
             # having 'key' as their parent
-            table = defaultdict(list)
+            table = collections.defaultdict(list)
             if ppid_map is None:
                 for p in process_iter():
                     try:
@@ -862,7 +882,7 @@ class Process(object):
             # from a process with multiple threads running on different
             # CPU cores, see:
             # http://stackoverflow.com/questions/1032357
-            # https://code.google.com/p/psutil/issues/detail?id=474
+            # https://github.com/giampaolo/psutil/issues/474
             overall_percent = ((delta_proc / delta_time) * 100) * num_cpus
         except ZeroDivisionError:
             # interval was too low
@@ -929,7 +949,7 @@ class Process(object):
                 except KeyError:
                     d[path] = nums
             nt = _psplatform.pmmap_grouped
-            return [nt(path, *d[path]) for path in d]
+            return [nt(path, *d[path]) for path in d]  # NOQA
         else:
             nt = _psplatform.pmmap_ext
             return [nt(*x) for x in it]
@@ -964,10 +984,13 @@ class Process(object):
 
     if _POSIX:
         def _send_signal(self, sig):
+            # XXX: according to "man 2 kill" PID 0 has a special
+            # meaning as it refers to <<every process in the process
+            # group of the calling process>>, so should we prevent
+            # it here?
             try:
                 os.kill(self.pid, sig)
-            except OSError:
-                err = sys.exc_info()[1]
+            except OSError as err:
                 if err.errno == errno.ESRCH:
                     self._gone = True
                     raise NoSuchProcess(self.pid, self._name)
@@ -1200,7 +1223,7 @@ class Popen(Process):
     def __init__(self, *args, **kwargs):
         # Explicitly avoid to raise NoSuchProcess in case the process
         # spawned by subprocess.Popen terminates too quickly, see:
-        # https://code.google.com/p/psutil/issues/detail?id=193
+        # https://github.com/giampaolo/psutil/issues/193
         self.__subproc = subprocess.Popen(*args, **kwargs)
         self._init(self.__subproc.pid, _ignore_nsp=True)
 
@@ -1254,6 +1277,7 @@ def pid_exists(pid):
 
 _pmap = {}
 
+
 def process_iter():
     """Return a generator yielding a Process instance for all
     running processes.
@@ -1444,6 +1468,7 @@ def cpu_times(percpu=False):
 _last_cpu_times = cpu_times()
 _last_per_cpu_times = cpu_times(percpu=True)
 
+
 def cpu_percent(interval=None, percpu=False):
     """Return a float representing the current system-wide CPU
     utilization as a percentage.
@@ -1528,6 +1553,7 @@ def cpu_percent(interval=None, percpu=False):
 _last_cpu_times_2 = _last_cpu_times
 _last_per_cpu_times_2 = _last_per_cpu_times
 
+
 def cpu_times_percent(interval=None, percpu=False):
     """Same as cpu_percent() but provides utilization percentages
     for each specific CPU time as is returned by cpu_times().
@@ -1558,7 +1584,7 @@ def cpu_times_percent(interval=None, percpu=False):
             if _WINDOWS:
                 # XXX
                 # Work around:
-                # https://code.google.com/p/psutil/issues/detail?id=392
+                # https://github.com/giampaolo/psutil/issues/392
                 # CPU times are always supposed to increase over time
                 # or at least remain the same and that's because time
                 # cannot go backwards.
@@ -1789,10 +1815,12 @@ def net_connections(kind='inet'):
     """
     return _psplatform.net_connections(kind)
 
+
 # =====================================================================
 # --- other system related functions
 # =====================================================================
 
+
 def boot_time():
     """Return the system boot time expressed in seconds since the epoch.
     This is also available as psutil.BOOT_TIME.
@@ -1883,7 +1911,6 @@ def test():
     output.
     """
     import datetime
-    from psutil._compat import print_
 
     today_day = datetime.date.today()
     templ = "%-10s %5s %4s %4s %7s %7s %-13s %5s %7s  %s"
@@ -1892,8 +1919,8 @@ def test():
     if _POSIX:
         attrs.append('uids')
         attrs.append('terminal')
-    print_(templ % ("USER", "PID", "%CPU", "%MEM", "VSZ", "RSS", "TTY",
-                    "START", "TIME", "COMMAND"))
+    print(templ % ("USER", "PID", "%CPU", "%MEM", "VSZ", "RSS", "TTY",
+                   "START", "TIME", "COMMAND"))
     for p in process_iter():
         try:
             pinfo = p.as_dict(attrs, ad_value='')
@@ -1930,16 +1957,17 @@ def test():
                 int(pinfo['memory_info'].rss / 1024) or '?'
             memp = pinfo['memory_percent'] and \
                 round(pinfo['memory_percent'], 1) or '?'
-            print_(templ % (user[:10],
-                            pinfo['pid'],
-                            pinfo['cpu_percent'],
-                            memp,
-                            vms,
-                            rss,
-                            pinfo.get('terminal', '') or '?',
-                            ctime,
-                            cputime,
-                            pinfo['name'].strip() or '?'))
+            print(templ % (
+                user[:10],
+                pinfo['pid'],
+                pinfo['cpu_percent'],
+                memp,
+                vms,
+                rss,
+                pinfo.get('terminal', '') or '?',
+                ctime,
+                cputime,
+                pinfo['name'].strip() or '?'))
 
 
 def _replace_module():
@@ -1979,7 +2007,7 @@ def _replace_module():
 
 
 _replace_module()
-del property, memoize, division, _replace_module
+del memoize, division, _replace_module
 if sys.version_info < (3, 0):
     del num
 
