@@ -1,14 +1,13 @@
 /*
- * $Id: process_info.c 1187 2011-10-22 15:56:59Z g.rodola $
- *
  * Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
  * Use of this source code is governed by a BSD-style license that can be
  * found in the LICENSE file.
  *
- * Helper functions related to fetching process information. Used by _psutil_bsd
- * module methods.
+ * Helper functions related to fetching process information.
+ * Used by _psutil_bsd module methods.
  */
 
+
 #include <Python.h>
 #include <assert.h>
 #include <errno.h>
@@ -34,10 +33,10 @@
  * On error, the function returns a BSD errno value.
  */
 int
-get_proc_list(struct kinfo_proc **procList, size_t *procCount)
+psutil_get_proc_list(struct kinfo_proc **procList, size_t *procCount)
 {
     int err;
-    struct kinfo_proc * result;
+    struct kinfo_proc *result;
     int done;
     static const int name[] = { CTL_KERN, KERN_PROC, KERN_PROC_PROC, 0 };
     // Declaring name as const requires us to cast it when passing it to
@@ -84,7 +83,7 @@ get_proc_list(struct kinfo_proc **procList, size_t *procCount)
         // error, toss away our buffer and start again.
         if (err == 0) {
             err = sysctl((int *) name, (sizeof(name) / sizeof(*name)) - 1,
-                          result, &length, NULL, 0);
+                         result, &length, NULL, 0);
             if (err == -1)
                 err = errno;
             if (err == 0) {
@@ -114,9 +113,9 @@ get_proc_list(struct kinfo_proc **procList, size_t *procCount)
 
 
 char
-*getcmdpath(long pid, size_t *pathsize)
+*psutil_get_cmd_path(long pid, size_t *pathsize)
 {
-    int  mib[4];
+    int mib[4];
     char *path;
     size_t size = 0;
 
@@ -134,21 +133,23 @@ char
     }
 
     path = malloc(size);
-    if (path == NULL)
+    if (path == NULL) {
+        PyErr_NoMemory();
         return NULL;
+    }
 
     *pathsize = size;
     if (sysctl(mib, 4, path, &size, NULL, 0) == -1) {
         free(path);
-        return NULL;       /* Insufficient privileges */
+        return NULL;       // Insufficient privileges
     }
 
     return path;
 }
 
 
-
 /*
+ * XXX no longer used; it probably makese sense to remove it.
  * Borrowed from psi Python System Information project
  *
  * Get command arguments and environment variables.
@@ -161,13 +162,13 @@ char
  *      1 for insufficient privileges.
  */
 char
-*getcmdargs(long pid, size_t *argsize)
+*psutil_get_cmd_args(long pid, size_t *argsize)
 {
-    int mib[4];
-    size_t size, argmax;
+    int mib[4], argmax;
+    size_t size = sizeof(argmax);
     char *procargs = NULL;
 
-    /* Get the maximum process arguments size. */
+    // Get the maximum process arguments size.
     mib[0] = CTL_KERN;
     mib[1] = KERN_ARGMAX;
 
@@ -175,10 +176,12 @@ char
     if (sysctl(mib, 2, &argmax, &size, NULL, 0) == -1)
         return NULL;
 
-    /* Allocate space for the arguments. */
+    // Allocate space for the arguments.
     procargs = (char *)malloc(argmax);
-    if (procargs == NULL)
+    if (procargs == NULL) {
+        PyErr_NoMemory();
         return NULL;
+    }
 
     /*
      * Make a sysctl() call to get the raw argument space of the process.
@@ -191,7 +194,7 @@ char
     size = argmax;
     if (sysctl(mib, 4, procargs, &size, NULL, 0) == -1) {
         free(procargs);
-        return NULL;       /* Insufficient privileges */
+        return NULL;       // Insufficient privileges
     }
 
     // return string and set the length of arguments
@@ -200,9 +203,9 @@ char
 }
 
 
-/* returns the command line as a python list object */
-PyObject*
-get_arg_list(long pid)
+// returns the command line as a python list object
+PyObject *
+psutil_get_arg_list(long pid)
 {
     char *argstr = NULL;
     int pos = 0;
@@ -214,30 +217,21 @@ get_arg_list(long pid)
         return retlist;
     }
 
-    // XXX - this leaks memory (grrr)
-    argstr = getcmdargs(pid, &argsize);
-
-    if (NULL == argstr) {
-        if (ESRCH == errno) {
-            PyErr_Format(PyExc_RuntimeError,
-                    "getcmdargs() failed - no process found with pid %lu", pid);
-            return NULL;
-        }
-
-        // ignore other errors for now, since we don't want to bail on
-        // get_process_info() if cmdline is the only thing we couldn't get.
-        // In that case, we just return an empty list return
-        // PyErr_Format(PyExc_RuntimeError, "getcmdargs() failed for pid %lu", pid);
-        return retlist;
+    argstr = psutil_get_cmd_args(pid, &argsize);
+    if (argstr == NULL) {
+        goto error;
     }
 
     // args are returned as a flattened string with \0 separators between
     // arguments add each string to the list then step forward to the next
     // separator
     if (argsize > 0) {
-        while(pos < argsize) {
+        while (pos < argsize) {
             item = Py_BuildValue("s", &argstr[pos]);
-            PyList_Append(retlist, item);
+            if (!item)
+                goto error;
+            if (PyList_Append(retlist, item))
+                goto error;
             Py_DECREF(item);
             pos = pos + strlen(&argstr[pos]) + 1;
         }
@@ -245,6 +239,13 @@ get_arg_list(long pid)
 
     free(argstr);
     return retlist;
+
+error:
+    Py_XDECREF(item);
+    Py_DECREF(retlist);
+    if (argstr != NULL)
+        free(argstr);
+    return NULL;
 }
 
 
@@ -252,7 +253,7 @@ get_arg_list(long pid)
  * Return 1 if PID exists in the current process list, else 0.
  */
 int
-pid_exists(long pid)
+psutil_pid_exists(long pid)
 {
     int kill_ret;
     if (pid < 0) {
